#version 430 core

layout (early_fragment_tests) in;

// Input variables
in VertexData
{
	vec3 normal_ws;
	vec3 position_ws;
	vec2 tex_coord;
	vec3 tangent_ws;
	vec3 bitangent_ws;
} inData;

// Output variables
layout (location = 0) out vec4 final_color;

// Data of the camera
layout (std140, binding = 0) uniform CameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};

// Random numbers for random sampling directions
layout (std140, binding = 4) uniform RandomFloats
{
	vec4 random_values[1024];
};

layout (binding = 0) uniform samplerCube cubemap_tex;
layout (binding = 1) uniform sampler2D mask_tex;

// Data of the material
uniform vec3 material_diffuse;
uniform vec3 material_fresnel;
uniform float material_roughness;

// Parameters of the computation
uniform int diffuse_sample_count;
uniform int specular_sample_count;
uniform bool mask_roughness;
uniform bool use_shiny_layer;
uniform bool use_shiny_layer_mask;

//-----------------------------------------------------------------------

void main()
{
	float mask_value = texture(mask_tex, inData.tex_coord).r;

	// Basic variables
	vec3 N = normalize(inData.normal_ws);
	if (!gl_FrontFacing)
		N = -N;
	vec3 T = normalize(inData.tangent_ws);
	vec3 B = normalize(inData.bitangent_ws);
	vec3 V = normalize(eye_position - inData.position_ws);		// This is 'Eye' variable in previous lectures
	
	// Task 1: Implement diffuse lighting
	//		Hints:
	//			Don't forget to clamp all dot products. Also make them greater than 0.001 to avoid divisions by zero.
	//			Don't forget to normalize L vector.
	//			Diffuse color of the material is in 'material_diffuse'.
	//			Environment is in 'cubemap_tex'. Use function textureLod(tex, coord, lod) to sample the texture.
	//		Experiments:
	//			Try increasing the number of samples (from GUI). Also when sampling 'cubemap_tex', try sampling
	//				higher mipmap levels. In both experiments, notice the quality of the image and rendering times.
	vec3 dif = vec3(0.0);
	for (int i = 0; i < diffuse_sample_count; i++)
	{
		//dif += ...;
	}

	// Task 2: Implement specular lighting
	//		Hints:
	//			Don't forget to normalize H vector.
	//			Use the functions that computes Fresnel and geom. atten. from the previous lecture.
	//		Experiments:
	//			Try increasing the number of samples (from GUI) and sampling higher mipmap levels.
	//			Also try changing roughness according to the mask texture (it is already sampled,
	//				the value is in 'mask_value' variable). Think about whether the output color
	//				fits into the enviroment.
	vec3 spe = vec3(0.0);
	for (int i = 0; i < specular_sample_count; i++)
	{
		//spe += ...;
	}

	// Task 3: Implement shiny layer. Feel free to experiment. Try semitransparent layer.
	//		Hints: Non-metals have their F0 very low, 0.04 is a good choice.
	if (use_shiny_layer)
	{
		//dif = ...;
		//spe = ...;
	}

	// Add all three parts of the color together
	vec3 final_light = dif + spe;
	final_color = vec4(final_light, 1.0);
}

//-----------------------------------------------------------------------